const fs = require("fs-extra");
const path = require("path");

const createStructure = (projectName, answers) => {
  const rootPath = path.join(process.cwd(), projectName);

  const folders = [];

  if (answers.addConfig) {
    folders.push("config");
  }
  if (answers.addUsers) {
    folders.push(
      "users",
      "users/controllers",
      "users/routes",
      "users/validators"
    );
  }
  if (answers.addAdmin) {
    folders.push(
      "admin",
      "admin/controllers",
      "admin/routes",
      "admin/validators"
    );
  }
  if (answers.addSchemas) {
    folders.push("models");
  }
  if (answers.addMiddleware) {
    folders.push("middleware");
    
  }
  if (answers.addHelper) {
    folders.push("helpers");
  }
  if (answers.addViews) {
    folders.push("views");
  }
  if (answers.addUtils) {
    folders.push("utils");
  }

  folders.forEach((folder) => {
    fs.ensureDirSync(path.join(rootPath, folder));
  });

  // Create main index.js file
  const indexPath = path.join(rootPath,  "index.js");
  const indexContent = `
const express = require('express');
const app = express();
const port = process.env.PORT || 3000;

app.use(express.json());
app.use(express.urlencoded({ extended: true }));

// Import routes
const routes = require('./router.js');
app.use('/api', routes);

app.listen(port, () => {
  console.log(\`Server is running on port \${port}\`);
});
  `;
  fs.writeFileSync(indexPath, indexContent.trim());

  if (answers.addConfig) {
    // Create config files
    const configPath = path.join(rootPath,  "config");
    const configContent = `
module.exports = {
  port: process.env.PORT || 3000,
  db: {
    host: process.env.DB_HOST || 'localhost',
    port: process.env.DB_PORT || 27017,
    name: process.env.DB_NAME || 'database_name'
  },
  JWT_AUTH_TOKEN:process.env.DB_HOST
};
    `;
    fs.writeFileSync(path.join(configPath, "config.js"), configContent.trim());

    const dbConfigContent = `
const mongoose = require('mongoose');
const config = require('./config');

mongoose.connect(\`mongodb://\${config.db.host}:\${config.db.port}/\${config.db.name}\`, {
  useNewUrlParser: true,
  useUnifiedTopology: true
}).then(() => {
  console.log('Connected to the database');
}).catch((err) => {
  console.error('Database connection error:', err);
});
    `;
    fs.writeFileSync(
      path.join(configPath, "dbConfig.js"),
      dbConfigContent.trim()
    );
  }

  if (answers.addEnv) {
    // Create .env file
    const envContent = `
PORT=3000
DB_HOST=localhost
DB_PORT=27017
DB_NAME=database_name
JWT_AUTH_TOKEN=1c9a584d77940018e673caf9c04be51ae1cb0387092367ea5adc12cd0af777d0
    `;
    fs.writeFileSync(path.join(rootPath, ".env"), envContent.trim());
  }

  if (answers.addGitIgnore) {
    // Create .gitignore file
    const gitIgnoreContent = `
node_modules/
.env
    `;
    fs.writeFileSync(
      path.join(rootPath, ".gitignore"),
      gitIgnoreContent.trim()
    );
  }

  if (answers.addDockerFile) {
    // Create Dockerfile
    const dockerFileContent = `
# Base image
FROM node:18

# Create app directory
WORKDIR /index.js

# A wildcard is used to ensure both package.json AND package-lock.json are copied
COPY package*.json ./

# Install app dependencies
RUN npm install -f

# Bundle app source
COPY . .
# COPY .env

# # envs
# ENV PORT 8080
# # expose working port
# EXPOSE $PORT

# Start the server using the production build
CMD [ "node", "index.js" ]
    `;

    fs.writeFileSync(
      path.join(rootPath, "Dockerfile"),
      dockerFileContent.trim()
    );
  }

  let mainRouter

  if (answers.addAdmin && answers.addUsers)
  {
       mainRouter = `
const express = require("express");
const router = express.Router();

const userRouter = require("./users/routes/routes");
const AdminRouter = require("./admin/routes/routes");

router.use("/user", userRouter);
router.use("/admin", AdminRouter);

module.exports = router;

  `;
  }

  if(answers.addAdmin &&  !answers.addUsers)
  {
           mainRouter = `
const express = require("express");
const router = express.Router();

const AdminRouter = require("./admin/routes/routes");

router.use("/admin", AdminRouter);

module.exports = router;

  `;

  }

  if(!answers.addAdmin && answers.addUsers)
  {
           mainRouter = `
const express = require("express");
const router = express.Router();

const userRouter = require("./users/routes/routes");

router.use("/user", userRouter);

module.exports = router;

  `;

  }

    fs.writeFileSync(path.join(rootPath, "router.js"), mainRouter.trim());

    if(answers.addMiddleware)
    {
      const middlewareFile = `
      const jwt = require("jsonwebtoken");
const config = require("./../config/config");

module.exports.verifyJWTToken = (request, response, next) => {
    try {
        let token = request.headers.authorization;
        if (!token) {
            return response
                .status(403)
                .json({
                    status: false,
                    message: "Invalid token or expired!",
                    data: null,
                });
        } else {
            jwt.verify(token, config.JWT_AUTH_TOKEN, async (err, result) => {
                if (err) {
                    return response
                        .status(401)
                        .json({
                            status: false,
                            message: "You are Not Authorize",
                            data: null,
                        });
                } else {
                    if (result) {
                        request.body.user = result;
                        return next();
                    } else {
                        return response
                            .status(401)
                            .json({
                                status: false,
                                message: "Invalid token or expired!",
                                data: null,
                            });
                    }
                }
            });
        }
    } catch (e) {
        return response
            .status(500)
            .json({
                status: false,
                message: "Invalid token or expired!",
                data: null,
            });
    }
};

      `;

            const middlewarePath = path.join(rootPath, "middleware");

          fs.writeFileSync(
            path.join(middlewarePath, "JWT.middleware.js"),
            middlewareFile.trim()
          );

    }

    if(answers.addUsers)
    {
      const routerFile = `
      const express = require("express");
      const router = express.Router();

      // Import all your routes here. This is an example: create auth.route.js and import it here.
       const authRouter = require("./auth.route");   
       router.use("/auth", authRouter);

       module.exports = router;

      `;
             const routes = path.join(rootPath, "users/routes");
       
              

             fs.writeFileSync(
               path.join(routes, "routes.js"),
               routerFile.trim()
             );


      const authRouteFile = `
      const express = require("express");
const router = express.Router();

const { verifyJWTToken } = require("../../middleware/jwt.middleware");
const authController = require("../controllers/auth.controller");
const authValidation = require("../validators/auth.validator");

router.post("/register",verifyJWTToken, authValidation.register, authController.register);



module.exports = router;

      `; 
      
        const authRoutes = path.join(rootPath, "users/routes");

        fs.writeFileSync(
          path.join(authRoutes, "auth.route.js"),
          authRouteFile.trim()
        );


    const authController = `
    module.exports.register = async (request, response, next) => {
    try {

    // Write your logic here        
        return response.json({
            status: true,
            message: "You have registered as an admin successfully",
            data: null,
        });
    } catch (e) {
        console.log(e);
        return response.status(500).json({
            status: false,
            message: "Something went wrong. Please try again",
            data: null,
        });
    }
};
    `; 
    
     const authControllers = path.join(rootPath, "users/controllers");

     fs.writeFileSync(
       path.join(authControllers, "auth.controller.js"),
       authController.trim()
     );


     const authValidations = `
     module.exports.register = async (request, response, next) => {
    // Write your logic here!!!
      };
     `;

         const authValidation = path.join(rootPath, "users/validators");

         fs.writeFileSync(
           path.join(authValidation, "auth.validator.js"),
           authValidations.trim()
         );
    }

    if(answers.addAdmin)
    {
       const routerFile = `
      const express = require("express");
      const router = express.Router();

      // Import all your routes here. This is an example: create auth.route.js and import it here.
       const authRouter = require("./auth.route");   
       router.use("/auth", authRouter);

       module.exports = router;

      `;
       const routes = path.join(rootPath, "admin/routes");

       fs.writeFileSync(path.join(routes, "routes.js"), routerFile.trim());

       const authRouteFile = `
      const express = require("express");
const router = express.Router();

const { verifyJWTToken } = require("../../middleware/jwt.middleware");
const authController = require("../controllers/auth.controller");
const authValidation = require("../validators/auth.validator");

router.post("/register", verifyJWTToken,authValidation.register, authController.register);



module.exports = router;

      `;

       const authRoutes = path.join(rootPath, "admin/routes");

       fs.writeFileSync(
         path.join(authRoutes, "auth.route.js"),
         authRouteFile.trim()
       );

       const authController = `
    module.exports.register = async (request, response, next) => {
    try {

    // Write your logic here        
        return response.json({
            status: true,
            message: "You have registered as an admin successfully",
            data: null,
        });
    } catch (e) {
        console.log(e);
        return response.status(500).json({
            status: false,
            message: "Something went wrong. Please try again",
            data: null,
        });
    }
};
    `;

       const authControllers = path.join(rootPath, "admin/controllers");

       fs.writeFileSync(
         path.join(authControllers, "auth.controller.js"),
         authController.trim()
       );

       const authValidations = `
     module.exports.register = async (request, response, next) => {
    // Write your logic here!!!
      };
     `;

       const authValidation = path.join(rootPath, "admin/validators");

       fs.writeFileSync(
         path.join(authValidation, "auth.validator.js"),
         authValidations.trim()
       );

    }


  console.log(`Project structure for ${projectName} has been created.`);
};

module.exports = { createStructure };
